@@ -11,13 +11,13 @@ from typing import Optional, Any, Dict, List, Tuple
 import time
 import struct
 from uuid import uuid1
+import sys
 
-import psycopg2 as pg
-from tacview_client.config import DB_URL, get_logger
+import asyncpg
 import uvloop
 
+from tacview_client.config import DB_URL, get_logger
 
-CON = None
 CLIENT = 'tacview-client'
 PASSWORD = '0'
 STREAM_PROTOCOL = "XtraLib.Stream.0"
@@ -40,38 +40,30 @@ LOG = get_logger()
 
 class Ref:  # pylint: disable=too-many-instance-attributes
     """Hold and extract Reference values used as offsets."""
+    __slots__ = ('session_id', 'lat', 'lon', 'title', 'datasource', 'author',
+                 'file_version', 'start_time', 'time_offset', 'all_refs',
+                 'time_since_last', 'obj_store')
     def __init__(self):
         self.session_id: Optional[int] = None
         self.lat: Optional[float] = None
         self.lon: Optional[float] = None
         self.title: Optional[str] = None
-        self.datasource: Optional[str] = None
-        self.author: Optional[str] = None
+        self.datasource: Optional[str]
+        self.file_version: Optional[float] = None
+        self.author: Optional[str]
         self.start_time: Optional[datetime] = None
         self.time_offset: float = 0.0
         self.all_refs: bool = False
         self.time_since_last: float = 0.0
-        self.diff_since_last: float = 0.0
         self.obj_store: Dict[int, ObjectRec] = {}
-        self.all_refs: bool = False
-        self.written: bool = False
-        self.time_since_last_events: float = 0.0
 
     def update_time(self, offset):
         """Update the refence time attribute with a new offset."""
         offset = float(offset[1:])
-        self.diff_since_last = offset - self.time_offset
-        self.time_since_last += self.diff_since_last
-        self.time_since_last_events += self.diff_since_last
+        self.time_since_last = offset - self.time_offset
         self.time_offset = offset
 
-    def update_db(self):
-        self.time_since_last = 0.0
-
-    def write_events_db(self):
-        self.time_since_last_events = 0.0
-
-    def parse_ref_obj(self, line):
+    async def parse_ref_obj(self, line):
         """
         Attempt to extract ReferenceLatitude, ReferenceLongitude or
         ReferenceTime from a line object.
@@ -99,51 +91,42 @@ class Ref:  # pylint: disable=too-many-instance-attributes
                 LOG.debug('Ref Author found...')
                 self.author = val[1].decode('UTF-8')
 
+            elif val[0] == b'FileVersion':
+                LOG.debug('Ref Author found...')
+                self.file_version = float(val[1])
+
             elif val[0] == b'RecordingTime':
                 LOG.debug('Ref time found...')
                 self.start_time = datetime.strptime(val[1].decode('UTF-8'),
                                               '%Y-%m-%dT%H:%M:%S.%fZ')
 
-            self.all_refs = all(f
-                                for f in [self.lat and self.lon and self.start_time])
-            if self.all_refs and not self.written:
+            self.all_refs = bool(self.lat and self.lon and self.start_time)
+            if self.all_refs and not self.session_id:
                 LOG.info("All Refs found...writing session data to db...")
-                sess_ser = self.ser()
-                sql = f"""INSERT into session (lat, lon, title, datasource, author, start_time)
-                        VALUES (%s, %s, %s, %s, %s, %s)
+                sql = """INSERT into session (lat, lon, title,
+                                datasource, author, file_version, start_time)
+                        VALUES ($1, $2, $3, $4, $5, $6, $7)
                         RETURNING session_id
                 """
-                with CON.cursor() as cur:
-                    cur.execute(sql, list(sess_ser.values()))
-                    self.session_id = cur.fetchone()[0]
-                CON.commit()
+                async with ASYNC_CON.acquire() as con:
+                    self.session_id = await con.fetchval(
+                        sql, self.lat, self.lon, self.title, self.datasource,
+                        self.author, self.file_version, self.start_time)
 
-                self.written = True
                 LOG.info("Session session data saved...")
         except IndexError:
             pass
 
-    def ser(self):
-        """Serialize relevant Session fields for export."""
-        return {
-            'lat': self.lat,
-            'lon': self.lon,
-            'title': self.title,
-            'datasource': self.datasource,
-            'author': self.author,
-            'start_time': self.start_time
-        }
-
-
-# @dataclass
+from dataclasses import dataclass
+@dataclass
 class ObjectRec:
-    __slots__ = [
+    __slots__ = (
         'id', 'tac_id', 'first_seen', 'last_seen', 'session_id', 'alive', 'Name',
         'Color', 'Country', 'grp', 'Pilot', 'Type', 'Coalition', 'lat', 'lon',
         'alt', 'roll', 'pitch', 'yaw', 'u_coord', 'v_coord', 'heading',
         'impacted', 'impacted_dist', 'parent', 'parent_dist', 'updates',
         'velocity_kts', 'secs_since_last_seen', 'written', 'cart_coords'
-    ]
+    )
 
     def __init__(
             self,
@@ -157,7 +140,7 @@ class ObjectRec:
         self.last_seen = last_seen
         self.session_id = session_id
 
-        self.alive: int = 1
+        self.alive: bool = True
         self.Name: Optional[str] = None
         self.Color: Optional[str] = None
         self.Country: Optional[str] = None
@@ -191,25 +174,21 @@ class ObjectRec:
         self.secs_since_last_seen = value - self.last_seen
         self.last_seen = value
 
-    def compute_velocity(self, time_since_last_frame: float) -> None:
+    def compute_velocity(self) -> None:
         """Calculate velocity given the distance from the last point."""
         new_cart_coords = get_cartesian_coord(self.lat, self.lon, self.alt)
-        # new_cart_coords = tuple((self.v_coord, self.u_coord, self.alt))
-        if self.cart_coords is not None and self.secs_since_last_seen and self.secs_since_last_seen > 0:
-            true_dist = compute_dist(new_cart_coords, self.cart_coords)
-            self.velocity_kts = (true_dist /
-                                 self.secs_since_last_seen) / 1.94384
+        if self.cart_coords and self.secs_since_last_seen and self.secs_since_last_seen > 0:
+            t_dist = compute_dist(new_cart_coords, self.cart_coords)
+            self.velocity_kts = (t_dist / self.secs_since_last_seen) / 1.94384
         self.cart_coords = new_cart_coords
 
-    def should_have_parent(self):
+    def should_have_parent(self) -> bool:
+        parented_types = ('weapon', 'projectile', 'decoy', 'container', 'flare')
         tval = self.Type.lower()
-        return any([
-            t in tval
-            for t in ['weapon', 'projectile',
-                      'decoy',
-                      'container',
-                      'flare']
-        ])
+        for t in parented_types:
+            if t in tval:
+                return True
+        return False
 
 
 def get_cartesian_coord(lat, lon, h):
@@ -231,7 +210,7 @@ def compute_dist(p_1, p_2):
                 (p_2[2] - p_1[2])**2)
 
 
-def determine_contact(rec, ref: Ref, type='parent'):
+async def determine_contact(rec, ref: Ref, type='parent'):
     """Determine the parent of missiles, rockets, and bombs."""
     if type not in ['parent', 'impacted']:
         raise ValueError("Type must be impacted or parent!")
@@ -244,15 +223,15 @@ def determine_contact(rec, ref: Ref, type='parent'):
         accpt_colors = ['Blue', 'Red'
                         ] if rec.Color == 'Violet' else [rec.Color]
 
-        query_filter = " (type not like ('%Decoy%')"\
-            " AND type not like ('%Misc%')"\
-            " AND type not like ('%Weapon%')"\
-            " AND type not like ('%Projectile%')"\
-            " AND type not like ('%Ground+Light+Human+Air+Parachutist%'))"
+        query_filter = " (type not like '%Decoy%'"\
+            " AND type not like 'Misc%'"\
+            " AND type not like 'Weapon%'"\
+            " AND type not like 'Projectile%'"\
+            " AND type not like 'Ground+Light+Human+Air+Parachutist%')"
 
     elif type == 'impacted':
         accpt_colors = ['Red'] if rec.Color == 'Blue' else ['Red']
-        query_filter = " type like ('%Air+%')"
+        query_filter = " type like 'Air+%'"
 
     else:
         raise NotImplementedError
@@ -264,9 +243,10 @@ def determine_contact(rec, ref: Ref, type='parent'):
         AND session_id = {rec.session_id}
     """
     try:
-        with CON.cursor() as cur:
-            cur.execute(query)
-            nearby_objs = cur.fetchall()
+        async with ASYNC_CON.acquire() as con:
+            nearby_objs = await con.fetch(query)
+            # cur.execute(query)
+            # nearby_objs = cur.fetchall()
     except Exception as err:
         LOG.info(query)
         raise err
@@ -275,7 +255,7 @@ def determine_contact(rec, ref: Ref, type='parent'):
     for nearby in nearby_objs:
         near = ref.obj_store[nearby[0]]
         if ((near.last_seen <= offset_min
-             and not (near.Type.startswith('Ground') and near.alive == 1))
+             and not (near.Type.startswith('Ground') and near.alive == True))
                 and (abs(near.alt - rec.alt) < 2000)
                 and (abs(near.lat - rec.lat) <= 0.0005)
                 and (abs(near.lon - rec.lon) <= 0.0005)):
@@ -301,7 +281,7 @@ def determine_contact(rec, ref: Ref, type='parent'):
     return closest
 
 
-def line_to_obj(raw_line: bytearray, ref: Ref) -> Optional[ObjectRec]:
+async def line_to_obj(raw_line: bytearray, ref: Ref) -> Optional[ObjectRec]:
     """Parse a textline from tacview into an ObjectRec."""
     # secondary_update = None
     if raw_line[0:1] == b"0":
@@ -309,20 +289,15 @@ def line_to_obj(raw_line: bytearray, ref: Ref) -> Optional[ObjectRec]:
 
     if raw_line[0:1] == b'-':
         rec = ref.obj_store[int(raw_line[1:], 16)]
-        rec.alive = 0
-        mark_dead(rec.id)
+        rec.alive = False
+        rec.updates += 1
 
         if 'Weapon' in rec.Type:
-            impacted = determine_contact(rec, type='impacted', ref=ref)
+            impacted = await determine_contact(rec, type='impacted', ref=ref)
             if impacted:
                 rec.impacted = impacted[0]
                 rec.impacted_dist = impacted[1]
-                sql = create_impact_stmt()
-                vals = (ref.session_id, rec.parent, rec.impacted, rec.id,
-                        ref.time_offset, rec.impacted_dist)
-                with CON.cursor() as cur:
-                    cur.execute(sql, vals)
-                CON.commit()
+                await insert_impact(rec, ref.time_offset)
         return rec
 
     comma = raw_line.find(b',')
@@ -382,10 +357,10 @@ def line_to_obj(raw_line: bytearray, ref: Ref) -> Optional[ObjectRec]:
     except Exception as err:
         raise err
 
-    rec.compute_velocity(ref.time_since_last)
+    rec.compute_velocity()
 
     if rec.updates == 1 and rec.should_have_parent():
-        parent_info = determine_contact(rec, type='parent', ref=ref)
+        parent_info = await determine_contact(rec, type='parent', ref=ref)
         if parent_info:
             rec.parent = parent_info[0]
             rec.parent_dist = parent_info[1]
@@ -393,54 +368,18 @@ def line_to_obj(raw_line: bytearray, ref: Ref) -> Optional[ObjectRec]:
     return rec
 
 
-def create_object_stmt():
-    return """INSERT into object (
-            tac_id, session_id, name, color, country, grp, pilot, type,
-            alive, coalition, first_seen, last_seen, lat, lon, alt, roll,
-            pitch, yaw, u_coord, v_coord, heading, velocity_kts, impacted,
-            impacted_dist, parent, parent_dist, updates
-        )
-        --VALUES($1, $2, $3, $4, $5, $6, $7, $8, $9, $10, $11, $12, $13, $14,
-        --   $15, $16, $17, $18, $19, $20, $21, $22, $23, $24, $25, $26, $27)
-
-        VALUES(%s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s,
-           %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s)
-
-        RETURNING id
-        """
+async def insert_impact(rec, impact_time):
+    sql = """INSERT into impact (session_id, killer, target,
+                    weapon, time_offset, impact_dist)
+                VALUES($1, $2, $3, $4, $5, $6)
+    """
+    vals = (rec.session_id, rec.parent, rec.impacted, rec.id,
+            impact_time, rec.impacted_dist)
+    async with ASYNC_CON.acquire() as con:
+        await con.execute(sql, *vals)
 
 
-def create_impact_stmt():
-    return """INSERT into impact (
-        session_id, killer, target, weapon, time_offset, impact_dist)
-        VALUES(%s, %s, %s, %s, %s, %s)
-        """
-
-def mark_dead(obj_id) -> None:
-    """Mark a single record as dead."""
-    with CON.cursor() as cur:
-        cur.execute(" UPDATE object SET alive = 0 WHERE id = %s;", [obj_id])
-    CON.commit()
-    # await DB.execute(f"""
-    #     UPDATE object SET alive = 0 WHERE id = {obj_id};
-    # """)
-
-    # """
-    # WITH src AS (
-    #     UPDATE serial_rate
-    #     SET rate = 22.53, serial_key = '0002'
-    #     WHERE serial_key = '002' AND id = '01'
-    #     RETURNING *
-    #     )
-    # UPDATE serial_table dst
-    # SET serial_key = src.serial_key
-    # FROM src
-    # -- WHERE dst.id = src.id AND dst.serial_key  = '002'
-    # WHERE dst.id = '01' AND dst.serial_key  = '002';
-    # """
-
-
-def create_single(obj):
+async def create_single(obj):
     """Insert a single newly create record to database."""
     vals = (obj.tac_id,
             obj.session_id,
@@ -457,11 +396,22 @@ def create_single(obj):
             obj.velocity_kts, obj.impacted, obj.impacted_dist,
             obj.parent, obj.parent_dist, obj.updates)
 
-    sql = create_object_stmt()
-    with CON.cursor() as cur:
-        cur.execute(sql, vals)
-        obj.id = cur.fetchone()[0]
-    CON.commit()
+    sql =  """INSERT into object (
+            tac_id, session_id, name, color, country, grp, pilot, type,
+            alive, coalition, first_seen, last_seen, lat, lon, alt, roll,
+            pitch, yaw, u_coord, v_coord, heading, velocity_kts, impacted,
+            impacted_dist, parent, parent_dist, updates
+        )
+        VALUES($1, $2, $3, $4, $5, $6, $7, $8, $9, $10, $11, $12, $13, $14,
+           $15, $16, $17, $18, $19, $20, $21, $22, $23, $24, $25, $26, $27)
+
+        --VALUES(%s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s,
+        --   %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s)
+
+        RETURNING id
+        """
+    async with ASYNC_CON.acquire() as con:
+        obj.id = await con.fetchval(sql, *vals)
     obj.written = True
 
 
@@ -473,18 +423,16 @@ class MaxIterationsException(Exception):
     """Throw this exception when max iters < total_iters."""
 
 
-class AsyncSocketReader:
+class AsyncStreamReader(Ref):
+    reader: asyncio.StreamReader
+    writer: asyncio.StreamWriter
     """Read from Tacview socket."""
     def __init__(self, host, port, debug=False):
-        self.host = host
-        self.port = port
-        self.reader: Optional[asyncio.StreamReader] = None
-        self.ref = Ref()
-        self.writer: Optional[asyncio.StreamWriter] = None
+        super().__init__()
+        self.host: str = host
+        self.port: int = port
         self.sink = "log/raw_sink.txt"
-        self.data = bytearray()
         self.debug = debug
-        self.msg = None
         if self.debug:
             open(self.sink, 'w').close()
 
@@ -501,16 +449,10 @@ class AsyncSocketReader:
                 LOG.info('Connection opened...sending handshake...')
                 self.writer.write(HANDSHAKE)
                 await self.reader.readline()
-
-                LOG.info('Connection opened...creating db and reading refs...')
-                while not self.ref.all_refs:
-                    obj = await self.read_stream()
-                    if obj[0:2] == b"0,":
-                        self.ref.parse_ref_obj(obj)
-                        continue
+                LOG.info('Connection opened with successful handshake...')
                 break
             except ConnectionError:
-                LOG.error('Connection attempt failed....retry in 3 sec...')
+                LOG.error('Connection attempt failed....retrying in 3 sec...')
                 await asyncio.sleep(3)
 
     async def read_stream(self):
@@ -522,9 +464,6 @@ class AsyncSocketReader:
         """Close the socket connection and reset ref object."""
         self.writer.close()
         await self.writer.wait_closed()
-        self.reader = None
-        self.writer = None
-        self.ref = Ref()
 
 
 class BinCopyWriter:
@@ -532,7 +471,7 @@ class BinCopyWriter:
     db_event_time: float = 0.0
     event_times: List = []
     insert: BytesIO
-    fmt_str: str = '>h ii ii id ii id id id id id id id id id id ii'
+    fmt_str: str = '>h ii ii id i? id id id id id id id id id id ii'
     copy_header = struct.pack('>11sii', b'PGCOPY\n\377\r\n\0', 0, 0)
     copy_trailer =  struct.pack('>h', -1)
     tbl_uuid = str(uuid1()).replace("-", '_')
@@ -563,21 +502,42 @@ class BinCopyWriter:
             WHERE row_number = 1
             ON CONFLICT (id)
             DO UPDATE SET session_id=EXCLUDED.session_id,
-                last_seen=EXCLUDED.last_seen, alive=EXCLUDED.alive,
-                lat=EXCLUDED.lat, lon=EXCLUDED.lon, alt=EXCLUDED.alt,
-                roll=EXCLUDED.roll, pitch=EXCLUDED.pitch, yaw=EXCLUDED.yaw,
-                u_coord=EXCLUDED.u_coord, v_coord=EXCLUDED.v_coord,
-                heading=EXCLUDED.heading, velocity_kts=EXCLUDED.velocity_kts,
-                updates=EXCLUDED.updates;
+                last_seen=EXCLUDED.last_seen,
+                alive=EXCLUDED.alive,
+                lat=EXCLUDED.lat,
+                lon=EXCLUDED.lon,
+                alt=EXCLUDED.alt,
+                roll=EXCLUDED.roll,
+                pitch=EXCLUDED.pitch,
+                yaw=EXCLUDED.yaw,
+                u_coord=EXCLUDED.u_coord,
+                v_coord=EXCLUDED.v_coord,
+                heading=EXCLUDED.heading,
+                velocity_kts=EXCLUDED.velocity_kts,
+                updates=EXCLUDED.updates
+            WHERE object.updates < EXCLUDED.updates;
 
             DROP INDEX tmp_idx;
-
             DROP TABLE "{tbl_uuid}";
         """
 
-    def __init__(self, dsn: str, min_insert_size: int = -1):
+        # """
+        # WITH src AS (
+        #     UPDATE serial_rate
+        #     SET rate = 22.53, serial_key = '0002'
+        #     WHERE serial_key = '002' AND id = '01'
+        #     RETURNING *
+        #     )
+        # UPDATE serial_table dst
+        # SET serial_key = src.serial_key
+        # FROM src
+        # -- WHERE dst.id = src.id AND dst.serial_key  = '002'
+        # WHERE dst.id = '01' AND dst.serial_key  = '002';
+        # """
+
+    def __init__(self, dsn: str, batch_size: int = 10000):
         self.dsn = dsn
-        self.min_insert_size = min_insert_size
+        self.batch_size = batch_size
         self.build_fmt_string()
         self.create_byte_buffer()
         self.insert_count = 0
@@ -589,9 +549,6 @@ class BinCopyWriter:
             "DOUBLE": ('id', 8),
             "NUMERIC": ('id', 8)
         }
-        # for col in columns.values():
-        #     fmt, sz = types[str(col.type)]
-        #     self.fmt_str.append(fmt)
 
     def create_byte_buffer(self) -> None:
         self.insert = BytesIO()
@@ -605,7 +562,7 @@ class BinCopyWriter:
                     4, obj.id,
                     4, obj.session_id,
                     8, obj.last_seen,
-                    4, obj.alive,
+                    1, obj.alive,
                     8, obj.lat,
                     8, obj.lon,
                     8, obj.alt,
@@ -626,84 +583,82 @@ class BinCopyWriter:
         self.insert.write(packed)
         self.insert_count += 1
 
-    def insert_data(self) -> None:
-        """If data is in buffer, execute binary copy and update."""
-        if self.min_insert_size > self.insert_count:
+    async def cleanup(self) -> None:
+        """Shut down and ensure all data is written."""
+        self.min_insert_size = -1 # ensure everything gets flushed
+        await self.insert_data(force=True)
+        self.db_event_time = sum(self.event_times)
+
+    async def insert_data(self, force=False) -> None:
+        if not force and self.batch_size > self.insert_count:
             LOG.debug("Not enough data for insert....")
             return
+
         LOG.debug(f'Inserting {self.insert_count} records...')
         self.insert.write(self.copy_trailer)
         self.insert.seek(0)
-        # conn = pg.connect(self.dsn)
-        with CON.cursor() as cur:
-            cur.copy_expert(self.cmd, self.insert)
-        # CON.commit()
+        async with ASYNC_CON.acquire() as con:
+            await con._copy_in(self.cmd , self.insert, 100)
         self.insert.close()
         self.create_byte_buffer()
 
-    def cleanup(self) -> None:
-        """Shut down and ensure all data is written."""
-        self.min_insert_size = -1 # ensure everything gets flushed
-        self.insert_data()
-        self.db_event_time = sum(self.event_times)
 
-
-async def consumer(host=HOST,
-                   port=PORT,
-                   max_iters=None,
-                   bulk=False,
-                   dsn:str = DB_URL) -> None:
+async def consumer(host: str,
+                   port: int,
+                   max_iters: Optional[int],
+                   batch_size: int,
+                   dsn:str) -> None:
     """Main method to consume stream."""
-    LOG.info("Starting consumer with settings: "
-             "debug: %s --  iters %s -- bulk-mode: %s", DEBUG, max_iters, bulk)
-    global CON
-    CON = pg.connect(dsn)
-    lines_read = 0
-    sock = AsyncSocketReader(host, port)
-    copy_writer = BinCopyWriter(dsn, 10)
+    LOG.info("Starting tacview client with settings: "
+             "debug: %s -- batch-size: %s", DEBUG, batch_size)
+    global ASYNC_CON
+    ASYNC_CON = await asyncpg.create_pool(DB_URL)
+    copy_writer = BinCopyWriter(dsn, batch_size)
+    sock = AsyncStreamReader(host, port)
     await sock.open_connection()
     init_time = time.time()
+    lines_read = 0
     last_log = float(0.0)
     line_proc_time = float(0.0)
-
     while True:
         try:
             obj = await sock.read_stream()
             lines_read += 1
+
+            if not sock.all_refs:
+                await sock.parse_ref_obj(obj)
+                continue
+
             if obj[0:1] == b"#":
-                sock.ref.update_time(obj)
-                if not bulk:
-                    copy_writer.insert_data()
+                sock.update_time(obj)
+                await copy_writer.insert_data()
 
                 runtime = time.time() - init_time
                 log_check = runtime - last_log
-                if log_check > 5 or obj == b"#0":
-                    secs_ahead = sock.ref.time_offset - (
-                        (time.time() - init_time))
-                    ln_sec = round(lines_read / runtime, 2)
-                    LOG.info(
-                        f"Runtime: {round(runtime, 2)} - Sec ahead: {round(secs_ahead, 2)}..."
-                        f"Lines/sec: {ln_sec} - Total: {lines_read}")
+                if log_check > 0.05:
+                    ln_sec = lines_read / runtime
+                    sys.stdout.write("\rEvents processed: {:,} at {:,.2f} events/sec".format(
+                        lines_read, ln_sec))
+                    sys.stdout.flush()
                     last_log = runtime
-
             else:
                 t1 = time.time()
-                obj = line_to_obj(obj, sock.ref)
+                obj = await line_to_obj(obj, sock)
+                line_proc_time += (time.time() - t1)
+                if not obj:
+                    continue
+                if not obj.written:
+                    await create_single(obj)
+                copy_writer.add_data(obj)
 
-                if obj:
-                    line_proc_time += (time.time() - t1)
-                    if not obj.written:
-                        create_single(obj)
-
-                    copy_writer.add_data(obj)
-
-            if max_iters and max_iters <= lines_read:
+            if max_iters and max_iters < lines_read:
+                await copy_writer.insert_data()
                 LOG.info(f"Max iters reached: {max_iters}...returning...")
                 raise MaxIterationsException
 
         except (KeyboardInterrupt, MaxIterationsException,
                 ServerExitException, asyncio.IncompleteReadError):
-            copy_writer.cleanup()
+            await copy_writer.cleanup()
             await sock.close()
             total_time = time.time() - init_time
             LOG.info('Total Lines Processed : %s', str(lines_read))
@@ -713,7 +668,7 @@ async def consumer(host=HOST,
             LOG.info('Pct Line Proc Time: %.2f', line_proc_time / total_time)
             LOG.info('Lines/second: %.4f', lines_read / total_time)
             total = {}
-            for obj in sock.ref.obj_store.values():
+            for obj in sock.obj_store.values():
                 if obj.should_have_parent() and not obj.parent:
                     try:
                         total[obj.Type] += 1
@@ -725,6 +680,11 @@ async def consumer(host=HOST,
             LOG.info('Exiting tacview-client!')
             return
 
+        except asyncpg.UniqueViolationError:
+                    LOG.error("The file you are trying to process is already in the database! "
+                              "To re-process it, delete all associated rows.")
+                    sys.exit(1)
+
         except Exception as err:
             LOG.error("Unhandled Exception!"
                       "Writing remaining updates to db and exiting!")
@@ -732,26 +692,23 @@ async def consumer(host=HOST,
             raise err
 
 
-def check_results():
+async def check_results():
     """Collect summary statistics on object and event records."""
-    with CON.cursor() as cur:
-        cur.execute(
+    con = await asyncpg.connect(DB_URL)
+    result = await con.fetchrow(
                 """SELECT COUNT(*) objects, COUNT(parent) parents,
                 (SELECT COUNT(*) FROM impact) impacts,
                 MAX(updates) max_upate, SUM(updates) total_updates,
                 (SELECT COUNT(*) events FROM event) as total_events,
-                SUM(alive) total_alive
+                COUNT(CASE WHEN alive THEN 1 END) total_alive
                 FROM object""")
-        result = cur.fetchone()
 
     print("Results:\nobjects: {} \nparents: {} \nimpacts: {}"
             "\nmax_updates: {} \ntotal updates: {}"
             "\ntotal events: {} \ntotal alive: {}".format(*list(result)))
-    CON.close()
+    await con.close()
 
-def main(host, port=42674, debug=False, max_iters=None, bulk=False, dsn=DB_URL):
+def main(host, port, max_iters, batch_size, dsn, debug=False):
     """Start event loop to consume stream."""
     uvloop.install()
-    loop = asyncio.get_event_loop()
-    asyncio.run(consumer(host, port, max_iters, bulk, dsn))
-
+    asyncio.run(consumer(host, port, max_iters, batch_size, dsn))
